from django.apps import apps


def user_logged_handler(sender, **kwargs):
    Message = apps.get_model(
        app_label='exo_messages',
        model_name='Message')

    user = kwargs.get('user')
    Message.all_objects.filter_by_user(
        user).filter_read_when_login().already_read().delete()
    Message.all_objects.filter_by_user(
        user).filter_read_when_login().not_read().mark_read()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-27 09:27
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.hstore
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields

import exo_messages.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'created', model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name='created',
                    ),
                ),
                (
                    'modified', model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name='modified',
                    ),
                ),
                ('can_be_closed', models.BooleanField(default=False)),
                ('read_when_login', models.BooleanField(default=False)),
                ('deleted', models.BooleanField(default=False)),
                ('read_at', models.DateTimeField(null=True)),
                ('description', models.TextField(blank=True)),
                (
                    'code', models.CharField(
                        choices=[
                            ('P', 'PENDING_VALIDATION_EMAIL'), (
                                'V',
                                'CONFIRMATION_VALIDATION_EMAIL',
                            ), ('W', 'PENDING_CHANGE_PASSWORD'),
                        ], max_length=1,
                    ),
                ),
                (
                    'level', models.IntegerField(choices=[
                        (10, 'debug'),
                        (20, 'info'), (25, 'success'), (30, 'warning'), (40, 'error'),
                    ]),
                ),
                ('variables', django.contrib.postgres.fields.jsonb.JSONField()),
                (
                    'user', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='messages', to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterModelManagers(
            name='message',
            managers=[
                ('objects', exo_messages.manager.MessageManager()),
            ],
        ),
    ]



from rest_framework import viewsets
from rest_framework import mixins
from rest_framework.permissions import IsAuthenticated
from rest_framework.decorators import detail_route
from rest_framework.response import Response

from ..models import Message
from .serializer import MessageSerializer


class MessageViewSet(
        mixins.ListModelMixin,
        mixins.RetrieveModelMixin,
        viewsets.GenericViewSet
):

    model = Message
    permission_classes = (IsAuthenticated,)
    serializer_class = MessageSerializer

    def get_queryset(self):
        return Message.objects.filter_by_user(
            self.request.user).filter_by_user_active()

    def get_all_queryset(self):
        return Message.all_objects.filter_by_user(self.request.user)

    @detail_route(methods=['post'], url_path='close')
    def close(self, request, pk):
        self.get_all_queryset().filter(pk=pk, can_be_closed=True).delete()
        return Response('ok')



from django.db.models import Manager
from django.core.exceptions import ObjectDoesNotExist

from .conf import settings
from .exceptions import ExoMessagesException
from .queryset import MessageQuerySet


class MessageManager(Manager):
    use_for_related_fields = True
    use_in_migrations = True
    queryset_class = MessageQuerySet

    variable_counter_name = 'counter'

    def get_queryset(self):
        return self.queryset_class(
            self.model,
            using=self._db).filter(
                deleted=False).filter_by_user_active()

    def filter_by_user(self, user):
        return self.queryset_class(self.model, using=self._db).filter(
            deleted=False,
            user=user,
        )

    def create_message(self, user, code, level, *args, **kwargs):
        if code not in [_[0] for _ in settings.EXO_MESSAGES_CH_CODE]:
            raise ExoMessagesException('Invalid code')
        if level not in [_[0] for _ in settings.EXO_MESSAGES_CH_LEVEL]:
            raise ExoMessagesException('Invalid level')

        variables = kwargs.get('variables', {})
        variables.update({'user_pk': user.pk})

        return self.create(
            user=user,
            code=code,
            level=level,
            can_be_closed=kwargs.get('can_be_closed', False),
            read_when_login=kwargs.get('read_when_login', False),
            variables=variables)

    def clear_messages(self, user, code):
        messages = self.filter_by_user(user).filter_by_code(code)
        messages.delete()

    def update_or_create_message(
            self, user, code, level,
            can_be_closed=False, read_when_login=False,
            **kwargs):
        """
            Creates or update a message for a given User/Code:
        """
        should_create_new_message = created = False
        message = None
        variables = kwargs.get('variables', {})

        if code not in [_[0] for _ in settings.EXO_MESSAGES_CH_CODE]:
            raise ExoMessagesException('Invalid code')
        if level not in [_[0] for _ in settings.EXO_MESSAGES_CH_LEVEL]:
            raise ExoMessagesException('Invalid level')

        try:
            message = self.filter_by_user(user).get(
                user=user,
                code=code,
                level=level,
                read_at__isnull=True)

            if variables.get(self.variable_counter_name, None):
                # Update counter
                variables_updated = message.variables
                new_counter_value = message.variables.get(
                    self.variable_counter_name,
                    0)
                new_counter_value = new_counter_value + variables.get(
                    self.variable_counter_name)
                variables_updated[self.variable_counter_name] = new_counter_value
                self.filter_by_user(user).filter(
                    code=code,
                    level=level,
                    read_at__isnull=True
                ).update(variables=variables_updated)

        except ObjectDoesNotExist:
            should_create_new_message = created = True

        if should_create_new_message:
            variables['user_pk'] = user.pk
            message = self.create_message(
                user=user,
                code=code,
                level=level,
                can_be_closed=can_be_closed,
                read_when_login=read_when_login,
                variables=variables)

        return message, created


class MessageAllManager(Manager):
    queryset_class = MessageQuerySet

    def get_queryset(self):
        return self.queryset_class(
            self.model,
            using=self._db,
        ).filter(deleted=False)

    def filter_by_user(self, user):
        return self.get_queryset().filter(user=user)



from django.db import models
from django.utils import timezone

from .conf import settings


class MessageQuerySet(models.QuerySet):

    def filter_read_when_login(self):
        return self.filter(read_when_login=True)

    def not_read(self):
        return self.filter(read_at__isnull=True)

    def already_read(self):
        return self.filter(read_at__isnull=False)

    def mark_read(self):
        self.update(read_at=timezone.now())

    def filter_by_code(self, code):
        return self.filter(code=code)

    def delete(self):
        self.update(deleted=True)

    def filter_by_user_active(self):
        default_statement = {
            models.When(
                user__is_active=True,
                then=models.Value(True),
            )
        }
        users_active_statement = getattr(
            settings,
            'CUSTOM_EXO_MESSAGE_USER_ACTIVE_STATEMENT',
            default_statement)

        status_case = models.Case(
            *users_active_statement,
            default=models.Value(False),
            output_field=models.BooleanField())
        return self.annotate(user_status=status_case).filter(user_status=True)



from django.db import models
from django.contrib.postgres.fields import JSONField
from django.utils import timezone

from model_utils.models import TimeStampedModel

from .conf import settings
from .manager import MessageManager, MessageAllManager


class Message(TimeStampedModel):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='messages',
        on_delete=models.deletion.CASCADE,
    )

    can_be_closed = models.BooleanField(default=False)
    read_when_login = models.BooleanField(default=False)

    description = models.TextField(blank=True)

    code = models.CharField(
        max_length=1,
        choices=settings.EXO_MESSAGES_CH_CODE,
    )
    level = models.IntegerField(
        choices=settings.EXO_MESSAGES_CH_LEVEL,
    )

    variables = JSONField()

    read_at = models.DateTimeField(null=True)
    deleted = models.BooleanField(default=False)

    objects = MessageManager()
    all_objects = MessageAllManager()

    def __str__(self):
        return '{}-{}'.format(self.user, self.get_code_display())

    def mark_as_read(self):
        self.read_at = timezone.now()
        self.save(update_fields=['read_at'])

    def delete(self, *args, **kwargs):
        self.deleted = True
        self.save(update_fields=['deleted'])

    def force_delete(self, *args, **kwargs):
        super().delete(*args, **kwargs)



# -*- coding: utf-8 -*-
"""
License boilerplate should be used here.
"""

# python 3 imports
from __future__ import absolute_import, unicode_literals

# python imports
import logging

# 3rd. libraries imports
from appconf import AppConf

# django imports
from django.conf import settings  # noqa

logger = logging.getLogger(__name__)


class ExoMessagesConfig(AppConf):

    CH_CODE_PENDING_EMAIL = 'P'
    CH_CODE_VALIDATED_EMAIL = 'V'
    CH_CODE_PENDING_PASSWORD = 'W'

    CH_CODE = (
        (CH_CODE_PENDING_EMAIL, 'PENDING_VALIDATION_EMAIL'),
        (CH_CODE_VALIDATED_EMAIL, 'CONFIRMATION_VALIDATION_EMAIL'),
        (CH_CODE_PENDING_PASSWORD, 'PENDING_CHANGE_PASSWORD'),
    ) + getattr(settings, 'ADITIONAL_EXO_MESSAGES_CODE', ())

    CH_DEBUG = 10
    CH_INFO = 20
    CH_SUCCESS = 25
    CH_WARNING = 30
    CH_ERROR = 40

    CH_LEVEL = (
        (CH_DEBUG, 'debug'),
        (CH_INFO, 'info'),
        (CH_SUCCESS, 'success'),
        (CH_WARNING, 'warning'),
        (CH_ERROR, 'error'),
    )



try:
    from django.conf.urls import url, include
except ImportError:
    from django.urls import re_path as url
    from django.urls import include

from rest_framework.routers import DefaultRouter

from .views import MessageViewSet

router = DefaultRouter()
router.register(r'messages', MessageViewSet, base_name='message')

app_name = 'exo_messages'

urlpatterns = [
    url(r'^', include(router.urls)),
]
